"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.transformPlugin = void 0;
const core_1 = /*#__PURE__*/require("@babel/core");
const helper_plugin_utils_1 = /*#__PURE__*/require("@babel/helper-plugin-utils");
const hash_1 = /*#__PURE__*/require("@emotion/hash");
const findUp = /*#__PURE__*/require("find-up");
const path_1 = /*#__PURE__*/require("path");
const fs_1 = /*#__PURE__*/require("fs");
const constants_1 = /*#__PURE__*/require("./constants");
/**
 * Checks that passed callee imports react context or context selector
 */
function isCreateContextCallee(path) {
  if (!path.isIdentifier) {
    return false;
  }
  return path.referencesImport(constants_1.REACT_PACKAGE, constants_1.CREATE_CONTEXT_CALL) || path.referencesImport(constants_1.CONTEXT_SELECTOR_PACKAGE, constants_1.CREATE_CONTEXT_CALL);
}
/**
 *
 * @param packageName - name of the package to import
 * @param functionName - name of the createContext function
 * @returns a new import expression for global context
 *
 * @example import { createContext as __createGlobalContext } from '@global-context/react'
 */
function createGlobalContextImportDeclaration(packageName, isContextSelector) {
  const origFunction = isContextSelector ? 'createContextSelector' : 'createContext';
  const newFunction = isContextSelector ? constants_1.GLOBAL_CONTEXT_SELECTOR_ALIAS : constants_1.GLOBAL_CONTEXT_ALIAS;
  return core_1.types.importDeclaration([core_1.types.importSpecifier(core_1.types.identifier(newFunction), core_1.types.identifier(origFunction))], core_1.types.stringLiteral(packageName));
}
/**
 * @returns - An expression that creates a global context
 *
 * @example const MyContext = __createGlobalContext();
 */
function createGlobalContextCallExpression(options) {
  const {
    expressionPath,
    packageJson,
    packageJsonPath,
    filePath,
    functionName
  } = options;
  const args = expressionPath.get('arguments').map(arg => arg.node);
  if (!expressionPath.parentPath.isVariableDeclarator()) {
    return expressionPath.node;
  }
  // Use the relative path from package.json because the same package
  // can be installed under different paths in node_modules if they are duplicated
  const relativePath = path_1.relative(packageJsonPath, filePath);
  const id = expressionPath.parentPath.get('id');
  return core_1.types.callExpression(core_1.types.identifier(functionName), [...args, core_1.types.stringLiteral(hash_1.default(`${relativePath}@${id.node.name}`)), core_1.types.stringLiteral(packageJson.name), core_1.types.stringLiteral(packageJson.version)]);
}
/**
 * Checks if import statement import createContext() from React
 */
function hasReactImport(path) {
  return path.node.source.value === constants_1.REACT_PACKAGE;
}
/**
 * Checks if import statement import createContext() from @fluentui/react-context-selector
 */
function hasContextSelectorImport(path) {
  return path.node.source.value === constants_1.CONTEXT_SELECTOR_PACKAGE;
}
exports.transformPlugin = /*#__PURE__*/helper_plugin_utils_1.declare(api => {
  api.assertVersion(7);
  return {
    name: 'global-context',
    pre() {
      this.importDeclarationPaths = [];
      this.nativeExpressionPaths = [];
      this.contextSelectorExpressionPaths = [];
    },
    visitor: {
      Program: {
        exit(path, state) {
          var _a;
          if (state.filename === undefined || !((_a = state.importDeclarationPaths) === null || _a === void 0 ? void 0 : _a.length) || !state.nativeExpressionPaths || !state.contextSelectorExpressionPaths) {
            return;
          }
          const packageJsonPath = findUp.sync('package.json', {
            cwd: path_1.dirname(state.filename)
          });
          if (packageJsonPath === undefined) {
            return;
          }
          if (state.importDeclarationPaths.some(hasReactImport)) {
            // Adds import for global context
            path.unshiftContainer('body', createGlobalContextImportDeclaration(constants_1.GLOBAL_CONTEXT_PACKAGE, false));
          }
          if (state.importDeclarationPaths.some(hasContextSelectorImport)) {
            // Adds import for global context selector
            path.unshiftContainer('body', createGlobalContextImportDeclaration(constants_1.GLOBAL_CONTEXT_PACKAGE, true));
          }
          const packageJson = JSON.parse(fs_1.readFileSync(packageJsonPath).toString());
          // substitutes expressions of react createContext to global context
          for (const expressionPath of state.contextSelectorExpressionPaths) {
            expressionPath.replaceWith(createGlobalContextCallExpression({
              expressionPath,
              packageJson,
              packageJsonPath,
              filePath: state.filename,
              functionName: constants_1.GLOBAL_CONTEXT_SELECTOR_ALIAS
            }));
          }
          for (const expressionPath of state.nativeExpressionPaths) {
            expressionPath.replaceWith(createGlobalContextCallExpression({
              expressionPath,
              packageJson,
              packageJsonPath,
              filePath: state.filename,
              functionName: constants_1.GLOBAL_CONTEXT_ALIAS
            }));
          }
        }
      },
      /**
       * Finds imports of `react` or `@fluentui/react-context-selector`.
       * If `createContext` is explicitly imported, stores its imported name in state
       *
       * @example import { createContext as createMyContext } from 'react';
       */
      // eslint-disable-next-line @typescript-eslint/naming-convention
      ImportDeclaration(path, state) {
        var _a;
        if (!hasReactImport(path) && !hasContextSelectorImport(path)) {
          return;
        }
        const native = hasReactImport(path);
        (_a = state.importDeclarationPaths) === null || _a === void 0 ? void 0 : _a.push(path);
        for (const importSpecifier of path.node.specifiers) {
          if (core_1.types.isImportSpecifier(importSpecifier) && core_1.types.isIdentifier(importSpecifier.imported) && core_1.types.isIdentifier(importSpecifier.local) && importSpecifier.imported.name === constants_1.CREATE_CONTEXT_CALL) {
            const localName = importSpecifier.local.name;
            if (native) {
              state.nativeLocalName = localName;
            } else {
              state.contextSelectorLocalName = localName;
            }
          }
        }
      },
      /**
       * Handles case when `createContext()` is `CallExpression`.
       *
       * @example createContext({})
       */
      // eslint-disable-next-line @typescript-eslint/naming-convention
      CallExpression(path, state) {
        var _a, _b, _c;
        if (((_a = state.importDeclarationPaths) === null || _a === void 0 ? void 0 : _a.length) === 0) {
          return;
        }
        const calleePath = path.get('callee');
        if (!isCreateContextCallee(calleePath)) {
          return;
        }
        if (core_1.types.isCallExpression(path.node) && core_1.types.isIdentifier(path.node.callee)) {
          if (path.node.callee.name === state.nativeLocalName) {
            (_b = state.nativeExpressionPaths) === null || _b === void 0 ? void 0 : _b.push(path);
          }
          if (path.node.callee.name === state.contextSelectorLocalName) {
            (_c = state.contextSelectorExpressionPaths) === null || _c === void 0 ? void 0 : _c.push(path);
          }
        }
      },
      /**
       * Handles case when `createContext()` is inside `MemberExpression`.
       * Assumes that context selector is not used this way
       *
       * @example module.createContext({})
       */
      // eslint-disable-next-line @typescript-eslint/naming-convention
      MemberExpression(expressionPath, state) {
        var _a;
        const objectPath = expressionPath.get('object');
        const propertyPath = expressionPath.get('property');
        const isCreateContextCall = objectPath.isIdentifier({
          name: 'React'
        }) && propertyPath.isIdentifier({
          name: 'createContext'
        });
        if (!isCreateContextCall) {
          return;
        }
        const parentPath = expressionPath.parentPath;
        if (!parentPath.isCallExpression()) {
          return;
        }
        (_a = state.nativeExpressionPaths) === null || _a === void 0 ? void 0 : _a.push(parentPath);
      }
    }
  };
});
//# sourceMappingURL=transformPlugin.js.map